@@ -1,6 +1,6 @@
 # alacorder beta 73 CLI
-
 from alacorder import alac
+import pandas as pd
 import click
 import os
 table = ""
@@ -146,6 +146,11 @@ def cli(path, output, archive, count, warn, bar, table, verbose, overwrite, laun
 
 	if outcheck == "archive" or outcheck == "existing_archive":
 		supportTable = False
+
+	if os.path.splitext(output)[1] == ".xls" or os.path.splitext(output)[1] == ".xlsx":
+		a = alac.config(path, table_path=output, table=table, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch)
+		click.echo(a)
+		b = alac.parseCases(a)
  
 	if supportArchive == False and (outcheck == "archive" or outcheck == "existing_archive"):
 		supportTable = False
@@ -155,9 +160,21 @@ def cli(path, output, archive, count, warn, bar, table, verbose, overwrite, laun
 	if supportTable == False and supportArchive == False:
 		click.echo("Failed to configure export!")
 
+	def getBool(y):
+		if isinstance(y, str):
+			if y == "":
+				return False
+			else:
+				return True
+		if isinstance(y, bool):
+			return bool(y)
+
+
 	if archive:
-		a = alac.config(path, archive_path=output, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch)
-		click.echo(a)
+		a = alac.config(path, archive_path=output, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch, mk_archive=True)
+		b = a.map(lambda x: getBool(x))
+		c = a[b == True]
+		click.echo(c)
 		b = alac.writeArchive(a)
 
 	if supportTable and (outcheck == "table" or outcheck == "overwrite_table"):
@@ -177,23 +194,16 @@ def cli(path, output, archive, count, warn, bar, table, verbose, overwrite, laun
 				click.echo("WARNING: Invalid table selection - defaulting to \'cases\'...")
 			table = "cases"
 		a = alac.config(path, table_path=output, table=table, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch)
+		b = a.map(lambda x: getBool(x))
+		c = a[b == True]
+		click.echo(c)
+		b = alac.parseTable(a)
+
+	elif supportTable and (outcheck == "all" or outcheck == "all_tables" or outcheck == "overwrite_all_tables"):
+		a = alac.config(path, table_path=output, table=table, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch)
 		click.echo(a)
 		b = alac.parseTable(a)
 
-		# if table != "all_table" and table != "all" and table != "cases" and table != "fees" and table != "charges" and table != "disposition" and table != "filing":
-		#	table = "cases"
-
-
-
-
-	if supportTable and (os.path.splitext(output)[1] == ".xls" or os.path.splitext(output)[1] == ".xlsx"):
-		try:
-			a = alac.config(path, table_path=output, table=table, GUI_mode=False, print_log=bar, warn=warn, verbose=verbose, max_cases=count, overwrite=overwrite, launch=launch)
-			click.echo(a)
-			b = alac.parseTable(a)
-			return b
-		except:
-			pass
 
 if __name__ == '__main__':
 	cli()
