@@ -18,8 +18,13 @@ import pandas as pd
 import time
 import warnings
 import click
+import inspect
 import PyPDF2
 from io import StringIO
+try:
+    import xlsxwriter
+except ImportError:
+    pass
 
 pd.set_option("mode.chained_assignment",None)
 pd.set_option("display.notebook_repr_html",True)
@@ -396,7 +401,7 @@ def getCharges(text: str):
     passed = passed.dropna()
     passed = pd.Series(passed.tolist())
     passed = passed.map(lambda x: re.sub(r'(\s+[0-1]{1}$)', '',x))
-    passed = passed.map(lambda x: re.sub(r'([©|\w|\#|\:\d]{1}[a-z]+)', ' ',x))
+    passed = passed.map(lambda x: re.sub(r'([©|\w]{1}[a-z]+)', ' ',x))
     passed = passed.explode()
     c = passed.dropna().tolist()
     cind = range(0, len(c))
@@ -553,8 +558,8 @@ def getConvictionCodes(text) -> [str]:
     return getCharges(text)[15]
 def getChargesString(text) -> str:
     return getCharges(text)[16]
-
-def config(input_path, table_path=None, archive_path=None, text_path=None, table="", print_log=True, verbose=False, warn=False, max_cases=0, overwrite=True, GUI_mode=False, drop_cols=True, repeat_duplicates=True, launch=False, no_write=False): 
+# config()
+def config(input_path, table_path=None, archive_path=None, text_path=None, table="", print_log=True, verbose=False, warn=False, max_cases=0, overwrite=True, GUI_mode=False, drop_cols=True, repeat_duplicates=True, launch=False, no_write=False, mk_archive=False): 
 
     tab_ext = ""
     arc_ext = ""
@@ -655,7 +660,8 @@ def config(input_path, table_path=None, archive_path=None, text_path=None, table
                 appendArchive = True
             except: 
                 pass
-
+        #else:
+         #   raise Exception("Invalid file extension! Archives must export to .pkl.xz")
 ## TABLE OUT 
     if table_path != None:
         tab_head = os.path.split(table_path)[0]
@@ -663,25 +669,17 @@ def config(input_path, table_path=None, archive_path=None, text_path=None, table
             raise Exception(f"Invalid table output path!")
         tab_tail = os.path.split(table_path)[1]
         tab_ext = os.path.splitext(tab_tail)[1]
-        if os.path.isfile(table_path) and overwrite == False:
+        if os.path.isfile(table_path):
             appendTable = True
             if tab_ext == ".xls" or tab_ext == ".xlsx":
                 try:
                     old_cases = pd.read_excel(table_path, sheet_name="cases")
-                except:
-                    old_cases = None
-                try:
                     old_fees = pd.read_excel(table_path, sheet_name="fees")
-                except:
-                    old_fees = None
-                try:
                     old_charges = pd.read_excel(table_path, sheet_name="charges")
-                except:
-                    old_charges = None
-                try:
                     old_table = [old_cases, old_fees, old_charges]
                 except:
-                    old_table = None
+                    appendTable = False
+                    pass
             elif tab_ext == ".json":
                 old_table = pd.read_json(table_path)
             elif tab_ext == ".csv":
@@ -714,7 +712,7 @@ def config(input_path, table_path=None, archive_path=None, text_path=None, table
         if content_length <= max_cases:
             click.echo(f"\n>>    INPUT:  {max_cases} {'paths' if pathMode else 'cases'} loaded from input: {input_path if pathMode else ''}")
         if table_path != None:
-            click.echo(f">>    table:  {'cases, charges, fees' if table == '' else table} to {table_path}")
+            click.echo(f">>    TABLE:  {'cases, charges, fees' if table == '' else table} to {table_path}")
         if archive_path != None:
             click.echo(f">>    ARCHIVE:  {'cases, charges, fees' if table == '' else table} to {'existing archive at: ' if appendArchive else ''}{archive_path}\n\n")
 ## OBJECT
@@ -738,7 +736,8 @@ def config(input_path, table_path=None, archive_path=None, text_path=None, table
         'path_mode': pathMode,
         'drop_cols': drop_cols,
         'launch': launch,
-        'no_write': no_write
+        'no_write': no_write,
+        'mk_archive': mk_archive
         })
 def splitext(path: str):
     head = os.path.split(path)[0]
@@ -787,8 +786,12 @@ def checkPath(path: str, log=True):
                     return PathType
                 else:
                     PathType = "overwrite_archive"
+                    if log:
+                        click.echo("WARNING: Existing file at archive output cannot be parsed and will be overwritten!")
                     return PathType
             elif ext == ".xls" or ext == ".xlsx":
+                if log:
+                    click.echo("WARNING: Existing file at archive output cannot be parsed and will be overwritten!")
                 PathType = "overwrite_all_table"
                 return PathType
             elif ext == ".csv" or ext == ".json" or ext == ".dta":
@@ -800,6 +803,8 @@ def checkPath(path: str, log=True):
                 PathType = "bad"
                 if log:
                     click.echo("Output file extension not supported!")
+                if log:
+                    click.echo("WARNING: Existing file at archive output cannot be parsed and will be overwritten!")
                 return PathType
         else:
             if ext == ".xls" or ext == ".xlsx":
@@ -821,15 +826,14 @@ def write(conf, outputs, archive=False):
     old_archive = conf['old_archive']
     old_table = conf['old_table']
     appendTable = conf['appendTable']
-    overwrite = conf['overwrite']
-    archive = conf['archive']
 
-    if overwrite == False and appendTable and isinstance(old_table, pd.core.frame.DataFrame):
+    if appendTable and isinstance(old_table, pd.core.frame.DataFrame):
         # print(outputs.info())
         out = [outputs, old_table]
         outputs = pd.concat(out)
+        print(outputs.info())
 
-    if overwrite == False and isinstance(old_archive, pd.core.frame.DataFrame):
+    if isinstance(old_archive, pd.core.frame.DataFrame):
         try:
             outputs = old_archive.append(outputs)
         except (AttributeError, TypeError):
@@ -866,7 +870,7 @@ def write(conf, outputs, archive=False):
                 with pd.ExcelWriter(path_out[0:-1]) as writer:
                     outputs.to_excel(writer, sheet_name="output-table")
             except ValueError:
-                outputs.to_csv(path_out,escapechar='\\')
+                outputs.to_csv(path_out+".csv",escapechar='\\')
                 if warn or print_log:
                     click.echo("Exported to CSV due to XLSX engine failure")
     elif out_ext == ".pkl":
@@ -884,7 +888,8 @@ def write(conf, outputs, archive=False):
     else:
         if warn:
             click.echo("Warning: Failed to export!")
-    return outputs 
+    size = os.path.getsize(path_out)
+    return size 
 def parseTable(conf, table=""):
     a = []
     if table == "all" or table == "all_cases" or table == "":
@@ -899,12 +904,6 @@ def parseTable(conf, table=""):
         a = parseCharges(conf)
     if table == "filing":
         a = parseCharges(conf)
-    if table == "consolidated":
-        ac = config(conf.input_path, table_path=conf.table_path+".xlsx", table="all", no_write=True)
-        a = parseCases(ac)
-        new = [a[0], a[1], a[2]] # cases fees charges
-        a = pd.concat(new)
-        aa = config(conf.input_path, table_path=conf.table_path, table="consolidated")
         write(aa, a)
     return a
 def writeArchive(conf): 
@@ -918,15 +917,11 @@ def writeArchive(conf):
     path_mode = conf['path_mode']
     max_cases = conf['count']
     old_archive = conf['old_archive']
+    overwrite = conf['overwrite']
     no_write = conf['no_write']
     start_time = time.time()
     if warn == False:
         warnings.filterwarnings("ignore")
-    allpagestext = pd.Series()
-    if print_log and path_mode:
-        with click.progressbar(queue) as bar:
-            for x in bar:
-                getPDFText(x)
 
     if path_mode:
         allpagestext = pd.Series(queue).map(lambda x: getPDFText(x))
@@ -1034,7 +1029,7 @@ def parseCharges(conf):
 
             b['CaseInfoOutputs'] = b['AllPagesText'].map(lambda x: getCaseInfo(x))
             b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
-            b['ChargesOutputs'] = b['AllPagesText'].map(lambda x: getCharges(x))
+            b['ChargesOutputs'] = b.index.map(lambda x: getCharges(b.loc[x].AllPagesText))
 
             
             chargetabs = b['ChargesOutputs'].map(lambda x: x[17])
@@ -1052,8 +1047,8 @@ def parseCharges(conf):
             if table == "disposition":
                 is_disp = charges.Disposition.map(lambda x: True if x == True else False)
                 charges = charges[is_disp]
-            if not no_write:
-                write(conf, charges)
+        if not no_write:
+            write(conf, charges)
     log_complete(conf, start_time)
 
     return charges
@@ -1067,7 +1062,6 @@ def parseCases(conf):
     warn = conf['warn']
     queue = conf['queue']
     appendTable = conf['appendTable']
-    overwrite = conf['overwrite']
     old_table = conf['old_table']
     no_write = conf['no_write']
     from_archive = False if conf['path_mode'] else True
@@ -1081,6 +1075,8 @@ def parseCases(conf):
     batchsize = max(pd.Series(batches).map(lambda x: x.shape[0]))
     if warn == False:
         warnings.filterwarnings("ignore")
+    temp_no_write_arc = False
+    temp_no_write_tab = False
     with click.progressbar(batches) as bar:
         for i, c in enumerate(bar):
             b = pd.DataFrame()
@@ -1097,7 +1093,7 @@ def parseCases(conf):
             b['Sex'] = b['CaseInfoOutputs'].map(lambda x: x[5])
             b['Address'] = b['CaseInfoOutputs'].map(lambda x: x[6])
             b['Phone'] = b['CaseInfoOutputs'].map(lambda x: x[7])
-            b['ChargesOutputs'] = b.AllPagesText.map(lambda x: getCharges(x))
+            b['ChargesOutputs'] = b.index.map(lambda x: getCharges(b.loc[x].AllPagesText))
             b['Convictions'] = b['ChargesOutputs'].map(lambda x: x[0])
             b['DispositionCharges'] = b['ChargesOutputs'].map(lambda x: x[1])
             b['FilingCharges'] = b['ChargesOutputs'].map(lambda x: x[2])
@@ -1118,6 +1114,7 @@ def parseCases(conf):
             b['TotalAmtDue'] = b['FeeOutputs'].map(lambda x: x[0])
             b['TotalBalance'] = b['FeeOutputs'].map(lambda x: x[1])
             b['PaymentToRestore'] = b['AllPagesText'].map(lambda x: getPaymentToRestore(x))
+            b['PaymentToRestore'][b.CERVConvictionCount == 0] = pd.NaT
             b['FeeCodesOwed'] = b['FeeOutputs'].map(lambda x: x[3])
             b['FeeCodes'] = b['FeeOutputs'].map(lambda x: x[4])
             b['FeeSheet'] = b['FeeOutputs'].map(lambda x: x[5])
@@ -1162,7 +1159,17 @@ def parseCases(conf):
             b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x,'coerce'))
             b['PaymentToRestore'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x,'coerce'))
 
-            if i % 5 == 0 or i == len(batches) - 1:
+            if bool(archive_out) and len(arc_ext) > 2 and i > 0 and not no_write:
+                if os.path.getsize(archive_out) > 1000:
+                    temp_no_write_arc = True
+            if bool(path_out) and i > 0 and not no_write:
+                if os.path.getsize(path_out) > 1000:
+                    temp_no_write_tab = True
+            if i == len(batches) - 1:
+                temp_no_write_arc = False
+                temp_no_write_tab = False
+
+            if (i % 5 == 0 or i == len(batches) - 1) and not no_write and temp_no_write_arc == False:
                 if bool(archive_out) and len(arc_ext) > 2:
                     timestamp = start_time
                     ar = pd.DataFrame({
@@ -1184,9 +1191,9 @@ def parseCases(conf):
             cases = cases.append(newcases, ignore_index=True)
             charges = charges[['CaseNumber', 'Num', 'Code', 'Description', 'Cite', 'CourtAction', 'CourtActionDate', 'Category', 'TypeDescription', 'Disposition', 'Permanent', 'Pardon', 'CERV','Conviction']]
             fees = fees[['CaseNumber', 'FeeStatus', 'AdminFee','Total', 'Code', 'Payor', 'AmtDue', 'AmtPaid', 'Balance', 'AmtHold']]
-
+            
             # write     
-            if appendTable == True and overwrite == False:
+            if appendTable:
                 if type(old_table) == list:
                     appcase = [cases, old_table[0]]
                     appcharge = [charges, old_table[1]]
@@ -1207,15 +1214,15 @@ def parseCases(conf):
                         cases = pd.concat(appcase)
 
 
-            if no_write == False and (i % 5 == 0 or i == len(batches) - 1):
+            if no_write == False and temp_no_write_tab == False and (i % 5 == 0 or i == len(batches) - 1):
                 if out_ext == ".xls":
                     try:
                         with pd.ExcelWriter(path_out,engine="xlsxwriter") as writer:
                             cases.to_excel(writer, sheet_name="cases")
                             fees.to_excel(writer, sheet_name="fees")
                             charges.to_excel(writer, sheet_name="charges")
-                    except (ImportError, ValueError):
-                        with pd.ExcelWriter(path_out) as writer:
+                    except (ImportError, IndexError, ValueError):
+                        with pd.ExcelWriter(path_out,engine="openpyxl") as writer:
                             cases.to_excel(writer, sheet_name="cases")
                             fees.to_excel(writer, sheet_name="fees")
                             charges.to_excel(writer, sheet_name="charges")
@@ -1225,24 +1232,21 @@ def parseCases(conf):
                             cases.to_excel(writer, sheet_name="cases")
                             fees.to_excel(writer, sheet_name="fees")
                             charges.to_excel(writer, sheet_name="charges")
-                    except (ImportError, ValueError):
+                    except (ImportError, IndexError, ValueError):
                         try:
-                            with pd.ExcelWriter(path_out) as writer:
+                            with pd.ExcelWriter(path_out,engine="openpyxl") as writer:
                                 cases.to_excel(writer, sheet_name="cases")
                                 fees.to_excel(writer, sheet_name="fees")
                                 charges.to_excel(writer, sheet_name="charges")
-                        except (ImportError, FileNotFoundError, ValueError):
+                        except (ImportError, FileNotFoundError, IndexError, ValueError):
                             try:
-                                cases.to_csv(path_out + ".csv",escapechar='\\')
-                                fees.to_csv(path_out + ".csv",escapechar='\\')
-                                charges.to_csv(path_out + ".csv",escapechar='\\')
+                                cases.to_csv(path_out + "-cases.csv",escapechar='\\')
+                                fees.to_csv(path_out + "-fees.csv",escapechar='\\')
+                                charges.to_csv(path_out + "-charges.csv",escapechar='\\')
                                 log_console(conf, f"(Batch {i+1}) - WARNING: Exported to CSV due to XLSX engine failure")
-                            except (ImportError, FileNotFoundError, ValueError):
+                            except (ImportError, FileNotFoundError, IndexError, ValueError):
+                                click.echo("Failed to export to CSV...")
                                 pass
-                elif out_ext == ".pkl":
-                    cases.to_pickle(path_out+".xz",compression="xz")
-                elif out_ext == ".xz":
-                    cases.to_pickle(path_out,compression="xz")
                 elif out_ext == ".json":
                     cases.to_json(path_out)
                 elif out_ext == ".csv":
@@ -1267,7 +1271,6 @@ def parseCaseInfo(conf):
     warn = conf['warn']
     queue = conf['queue']
     appendTable = conf['appendTable']
-    overwrite = conf['overwrite']
     from_archive = False if conf['path_mode'] else True
     start_time = time.time()
     arc_ext = conf['archive_ext']
@@ -1303,6 +1306,7 @@ def parseCaseInfo(conf):
             b['TotalBalance'] = b['Totals'].map(lambda x: x[3])
             b['TotalAmtHold'] = b['Totals'].map(lambda x: x[4])
             b['PaymentToRestore'] = b['AllPagesText'].map(lambda x: getPaymentToRestore(x))
+            b['PaymentToRestore'][b.CERVConvictionCount == 0] = pd.NaT
             b['ConvictionCodes'] = b['AllPagesText'].map(lambda x: getConvictionCodes(x))
             b['ChargeCodes'] = b['AllPagesText'].map(lambda x: getChargeCodes(x))
             b['FeeCodes'] = b['AllPagesText'].map(lambda x: getFeeCodes(x))
@@ -1333,7 +1337,6 @@ def parse(conf, method, *args):
     warn = conf['warn']
     queue = conf['queue']
     no_write = conf['no_write']
-    overwrite = conf['overwrite']
     from_archive = False if conf['path_mode'] else True
     if warn == False:
         warnings.filterwarnings("ignore")
@@ -1343,9 +1346,34 @@ def parse(conf, method, *args):
     start_time = time.time()
     alloutputs = []
     uselist = False
+    func = pd.Series(args).map(lambda x: 1 if inspect.isfunction(x) else 0)
+    funcs = func.index.map(lambda x: args[x] if func[x]>0 else np.nan)
+    funcs = funcs.dropna()
+    no_funcs = func.index.map(lambda x: args[x] if func[x]==0 else np.nan)
+    no_funcs = no_funcs.dropna()
+    countfunc = func.sum()
+    column_getters = pd.DataFrame(columns=['Name','Method','Arguments'],index=(range(0,countfunc)))
+    df_out = pd.DataFrame()
 
-    def ExceptionWrapper(mfunc, x, *args):
-        a = mfunc(x, *args)
+    local_get = []
+
+
+    for i, x in enumerate(funcs):
+        if inspect.isfunction(x):
+            column_getters.Name[i] = x.__name__
+            column_getters.Method[i] = x
+
+    for i, x in enumerate(args):
+        if inspect.isfunction(x) == False:
+            column_getters.Arguments.iloc[i-1] = x
+
+    def ExceptionWrapperArgs(mfunc, x, *args):
+        unpacked_args = args
+        a = mfunc(x, unpacked_args)
+        return a
+
+    def ExceptionWrapper(mfunc, x):
+        a = mfunc(x)
         return a
 
     with click.progressbar(batches) as bar:
@@ -1357,20 +1385,35 @@ def parse(conf, method, *args):
                 allpagestext = c
             else:
                 allpagestext = pd.Series(c).map(lambda x: getPDFText(x))
+            df_out['CaseNumber'] = allpagestext.map(lambda x: getCaseNumber(x))
+            for i, getter in enumerate(column_getters.Method.tolist()):
+                arg = pd.Series(column_getters.Arguments.tolist()[i])
+                #try:
+                if not arg.any():
+                    name = getter.__name__
+                    col = pd.DataFrame({
+                        name: allpagestext.map(lambda x: ExceptionWrapper(getter, x))
+                        })
+                else:
+                    name = getter.__name__
+                    col = pd.DataFrame({
+                        name: allpagestext.map(lambda x: ExceptionWrapperArgs(getter, x, arg))
+                        })
+                n_out = [df_out, col]
+                df_out = pd.concat(n_out,axis=1)
 
-            customoutputs = allpagestext.map(lambda x: ExceptionWrapper(method, x, *args))
-            alloutputs += customoutputs.tolist()
             if no_write == False and (i % 5 == 0 or i == len(batches) - 1):
-                write(conf, pd.Series(alloutputs))
+                write(conf, df_out) # rem alac
     if not no_write:
-        write(conf, pd.Series(alloutputs))
-    allout = pd.Series(alloutputs).infer_objects()
+        write(conf, df_out) # rem alac
+    allout = df_out.infer_objects()
     try:
         allout = allout.map(lambda x: x.values[0])
     except AttributeError:
         pass
     log_complete(conf, start_time)
     return allout
+
 ## LOG
 def log_complete(conf, start_time, output=None):
     path_in = conf['input_path']
